/*
  Tiny tidy

  T. Nakagawa
  2008/10/31
*/


#include <stdio.h>
#include <string.h>


#define BUF_SIZE (64 * 1024)


extern const char *in_word_set_inlinetag(char *, unsigned int);
extern const char *in_word_set_selfclosingtag(char *, unsigned int);
extern const char *in_word_set_specialblocktag(char *, unsigned int);


int main(int argc, char **argv) {
  int c, cc;
  FILE *rfp, *wfp;
  char buf[BUF_SIZE], *ptr = NULL;
  int inside, tagname, inlinetag, selfclosingtag, specialblocktag;

  if (argc < 2) return 1;
  rfp = fopen(argv[argc - 1], "rt");
  if (rfp == NULL) return 1;
  wfp = tmpfile();
  if (wfp == NULL) return 1;

  inside = 0;
  tagname = 0;
  inlinetag = 0;
  selfclosingtag = 0;
  specialblocktag = 0;
  while ((c = getc(rfp)) != EOF) {
    if (c == '<') {
      inside = 1;
      tagname = 1;
      ptr = buf;
    } else if (tagname && (c == ' ' || c == '\n' || c == '>')) {
      if (ptr == buf + BUF_SIZE) return 1;
      *(ptr++) = '\0';
      ptr = (*buf == '/') ? (buf + 1) : buf;
      tagname = 0;
      inlinetag = (in_word_set_inlinetag(ptr, strlen(ptr)) != NULL);
      selfclosingtag = (in_word_set_selfclosingtag(ptr, strlen(ptr)) != NULL);
      specialblocktag = (in_word_set_specialblocktag(ptr, strlen(ptr)) != NULL);
      if (!inlinetag && !(!specialblocktag && *buf == '/')) putc('\n', wfp);
      putc('<', wfp);
      fputs(buf, wfp);
      if (c != '\n') putc(c, wfp);
    } else if (tagname) {
      *(ptr++) = c;
    } else if (c == '\n' && inside) {
    } else {
      putc(c, wfp);
    }
    if (c == '>') {
      inside = 0;
      if ((!inlinetag && !(!specialblocktag && *buf != '/')) || selfclosingtag) putc('\n', wfp);
    }
  }

  fclose(rfp);
  rewind(wfp);
  rfp = wfp;
  wfp = fopen(argv[argc - 1], "wb");
  if (wfp == NULL) return 1;

  cc = '\n';
  while ((c = getc(rfp)) != EOF) {
    if (c == '\n' && cc == '\n') continue;
    putc(c, wfp);
    cc = c;
  }

  fclose(rfp);
  fclose(wfp);

  return 0;
}
